#include <plib.h>
#include <p32xxxx.h>
#include "./App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 15 ) / 1
// PB Clock = CPU Clock / 1
// Disabld WDT , ICSP/ICD = PGC4 , PGD4
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_15 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_8
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx4

#define SystemFrequency		60000000L
#define PBFrequency		SystemFrequency / 8
#define Timer1Tick		( PBFrequency / 256 / TogglesPerSec )
#define TogglesPerSec		1

const unsigned char LCMString1[ ] = "PIC32MX250F128D ";
const unsigned char LCMString2[ ] = "Rx:             ";
unsigned char UARTString1[ 32 ];
unsigned char Timer1TickFlag = 0;
unsigned char UART_IsReceived = 0;
unsigned char UART_ReceiveBuffer = 0;
unsigned char Hours = 0, Mins = 0, Secs = 0;

#define UARTChannel UART2	/* UART2 for APP1632 */

void UART_PutROMString(unsigned char * String);

void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    Timer1TickFlag++;

    // 清除Timer中斷旗標。
    mT1ClearIntFlag();

    // LED狀態反向。
    mPORTAToggleBits(BIT_9);
    mPORTCToggleBits(BIT_3);
}

void __ISR(_UART_2_VECTOR, ipl5) ISR_UART2(void)
{
    // 使用C32 UART Function讀取接收到的資料。
    //		   注意 : 此處資料僅簡單使用單一變數儲存，若資料瞬間流量大時，
    //		          必須透過其他軟體機制輔助，以免資料遺失，
    //		          例如:建立一個Data Queue來存放收到的資料。
    UART_ReceiveBuffer = UARTGetDataByte(UARTChannel);

    UART_IsReceived = 1;

    // 清除UART 接收中斷旗標。
    INTClearFlag(INT_U2RX);
}

int main(void)
{
    unsigned char Cursor = 0;

    // disable JTAG port
    DDPCONbits.JTAGEN = 0;

    // 周邊時脈 = 系統時脈 / 8。
    OSCSetPBDIV(OSC_PB_DIV_8);

    // 使用C32的SYSTEM Config巨集將CPU的操作最佳化，但不修改到FPBDIV的設定。
    SYSTEMConfig(SystemFrequency, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);
    // SYSTEMConfigPerformance( SystemFrequency  );
    // SYSTEMConfigWaitStates( SystemFrequency );

    // LED對應之I/O Port設定成數位輸出，並指定I/O Port狀態。
    mPORTASetPinsDigitalOut(BIT_9);
    mPORTCSetPinsDigitalOut(BIT_3);
    mPORTAClearBits(BIT_9);
    mPORTCSetBits(BIT_3);

    // 啟用Timer，並指定其中斷優先權。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4);

    // Timer設定。
    OpenTimer1(T1_ON | T1_IDLE_CON | T1_GATE_OFF | T1_PS_1_256 | T1_SOURCE_INT, Timer1Tick);

    // LCM初始化。
    LCM_Init();

    // 設定遊標，輸出字串。
    LCM_SetCursor(0, 0);
    LCM_PutROMString(LCMString1);
    LCM_SetCursor(0, 1);
    LCM_PutROMString(LCMString2);

    // 使用XC32 PPS Function指定UART Tx, Rx的腳位。
    // 注意 : 設定前記得先解鎖PPS, 設定為後再上鎖。
    PPSUnLock;
    PPSOutput(4, RPC9, U2TX); //Assign U2TX to pin RPC9
    PPSInput(2, U2RX, RPC8); //Assign U2RX to pin RPC8
    PPSOutput(1, RPB15, U2RTS); //Assign U2RTS to pin RPB15
    PPSInput(3, U2CTS, RPC6); //Assign U2CTS to pin RPC6
    PPSLock;

    // Disable RPB15 類比功能
    ANSELBbits.ANSB15 = 0;
    
    UARTConfigure(UARTChannel, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetFifoMode(UARTChannel, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UARTChannel, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UARTChannel, PBFrequency, 9600);
    UARTEnable(UARTChannel, UART_ENABLE | UART_PERIPHERAL | UART_TX | UART_RX);

    INTSetVectorPriority(INT_UART_2_VECTOR, INT_PRIORITY_LEVEL_5);
    INTSetVectorSubPriority(INT_UART_2_VECTOR, INT_SUB_PRIORITY_LEVEL_0);
    INTEnable(INT_U2RX, INT_ENABLED);

    Cursor = 3;

    INTEnableSystemMultiVectoredInt();

    while (1)
    {
        // UART收到資料 ?
        if (UART_IsReceived)
        {
            UART_IsReceived = 0;

            // 設定遊標，輸出字串。
            LCM_SetCursor(Cursor++, 1);
            LCM_PutASCII(UART_ReceiveBuffer);

            if (Cursor > 15) Cursor = 3;
        }

        // 時鐘處理。
        if (Timer1TickFlag >= 1)
        {
            if (++Secs > 59)
            {
                Secs = 0;
                if (++Mins > 59)
                {
                    Mins = 0;
                    if (++Hours > 23)
                        Hours = 0;
                }
            }

            sprintf(UARTString1, "Now Time : %02d:%02d:%02d\r\n",
                    Hours, Mins, Secs);

            // 使用C32 I/O UART Function將字串透過UART輸出。
            UART_PutROMString(UARTString1);

            Timer1TickFlag--;
        }
    }

    while (1);
}

void UART_PutROMString(unsigned char * String)
{
    while (*String != 0x00)
    {
        while (!UARTTransmitterIsReady(UARTChannel));
        UARTSendDataByte(UARTChannel, *String++);
    }
}
